/**
 * @fileOverview
 * @name sip.js
 * @author Wagner Müller Gegler, Roberto Oppermann Cordoni
 */

/**
 * @typedef {Object} SIPController~GeneralSection - General section parameters.
 * @property {String} allowoverlap - [yes|no] Enable/disable overlap dialing support.
 * @property {Number} bindport - UDP Port to bind to (listen on).
 * @property {String[]} codecs - Codecs to be allowed.
 * @property {String[]} includes - Path to files to be included in sip.conf.
 * @property {String[]} registers - Insert register definitions.
 * @property {String} srvlookup - [yes|no] Enables DNS SRV lookups on calls.
 * @property {String} t38pt_udptl - Set T.38 options.
 * @property {String} transport - Set transport protocol.
 */

/**
 * @typedef {Object} SIPController~AccountSection - Account section parameters.
 * @property {String} callerid - Caller ID information.
 * @property {String[]} codecs - Codecs to be allowed.
 * @property {String} context -  The Context for the call from this SIP user definition.
 * @property {String} username - Username.
 * @property {String} dtmfmode - [rfc2833|inband|info] Information on how the client handles DTMF signalling.
 * @property {String} host - Information on how to find the client.
 * @property {String[]} includes - Path to files to be included in sip.conf.
 * @property {String} name - Name of the peer.
 * @property {Number} port - SIP port.
 * @property {String} register - [enabled|disabled] Enable/disable registers.
 * @property {String} registers - Inform register definitions.
 * @property {String} secret - The user's password.
 * @property {String} type - [user|peer|friend] Relationship to client.
 */

/**
 * @class SIPController
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @property {SIPController~GeneralSection} general - General section parameters.
 * @property {SIPController~AccountSection[]} account - Account section parameters.
 * @property {String} textfile - Plaintext of sip.conf file.
 */
angular.module('ag561app').controller('SIPController', ['$http', '$log', '$location',
                                   function($http, $log, $location) {
  var url_sip = '/api/config/sip';
  var ctrl = this;
  ctrl.valid_bindport = true;
  ctrl.valid_port = true;
  ctrl.option = 'none';
  ctrl.invalid_config = 0;

  /**
   * Clear alert message from web page, but do this only
   * if we aren't with some error. We just want to erase
   * the message of danger if all the variables are correct.
   *
   * @function clear_alert_messages
   * @memberof SIPController
   */
  ctrl.clear_alert_messages = function() {
    if (!ctrl.invalid_config) {
      ctrl.alert_danger = false;
      ctrl.alert_success = false;
    }
  };

  /**
   * Get general and accounts section from sip.conf file.
   * @function get
   * @memberof SIPController
   */
  ctrl.get = function() {
    $http({
      method: 'GET',
      url: url_sip
      }).then(function(response) {
        // First function handles success
        ctrl.sip = response.data;
        for (var i=0; i < ctrl.sip.peers.length; i++) {
          ctrl.add_codecs_options(ctrl.sip.peers[i]);
        }
      }, function(response) {
        // Second function handles error
        ctrl.error = response.data;
      });
  };

  ctrl.get();

  /**
   * Save the new configuration.
   * If some error is happening then show in
   * web page.
   *
   * @function save
   * @memberof SIPController
   */
  ctrl.save = function() {

    for (i = 0; i < ctrl.sip.peers.length; i++) {
      ctrl.sip.peers[i].codecs = []; /* Clear list before do append  */

      for (var j = 0; j < ctrl.sip.peers[i].codecs_models.lists.Enabled.length; j++) {
        ctrl.sip.peers[i].codecs.push(ctrl.sip.peers[i].codecs_models.lists.Enabled[j].label);
      }

      // Everything is ok, then we need remove the variables that can not go tho API
      delete ctrl.sip.peers[i].view;
      delete ctrl.sip.peers[i].valid_port;
      delete ctrl.sip.peers[i].valid_username;
      delete ctrl.sip.peers[i].valid_host_ip;
      delete ctrl.sip.peers[i].valid_peer_name;
      delete ctrl.sip.peers[i].valid_bindport;
      delete ctrl.sip.peers[i].codecs_models;

      // Update string to context
      ctrl.sip.peers[i].context = "from-sip-".concat(ctrl.sip.peers[i].name);
    }

    ctrl.loading = true;
    var data = ctrl.sip;
    $http({
      method: 'PUT',
      url: url_sip,
      data: data,
      }).then(function(response) {
        // First function handles success
        ctrl.loading = false;
        ctrl.alert_success = true;
        ctrl.get();
      }, function(response) {
        // Second function handles error
        ctrl.loading = false;
        ctrl.alert_danger = true;
      });
  };

  /**
   * Save a object to remove in next step.
   *
   * @function save_peer_to_remove
   * @memberof SIPController
   */
  ctrl.save_peer_to_remove = function(array_index) {
    ctrl.peer_toremove = array_index;
  };

  /**
   * Remove now the object 'peer_toremove' from main object 'ctrl.sip.peers'
   *
   * @function remove_peer
   * @memberof SIPController
   */
  ctrl.remove_peer = function() {
    ctrl.sip.peers.splice(ctrl.peer_toremove, 1);
    ctrl.check_invalid_config();
  }

  function find_peer_name(name) {

    var i;

    for (i = 0; i < ctrl.sip.peers.length; i++) {
      if (ctrl.sip.peers[i].name == name)
        return ctrl.sip.peers[i];
    }

    return false;
  };

  /**
   * Start a new object for create a new peer.
   * Initializes default values for the new peer.
   *
   * @function add_peer
   * @memberof SIPController
   */
  ctrl.add_peer = function() {
    ctrl.clear_alert_messages();
    var i = 0;
    var peer_name;

    // Check if peer name already exists
    do {
      i++;
      peer_name = 'peer' + i.toString();
    } while (find_peer_name(peer_name));

    // Start default configuration
    var newpeer = {'codecs': ['alaw'],
                    'port': '5060',
                    'register': 'no',
                    'host': '',
                    'valid_host_ip': false, /* This variable is necessary to force user to write a valid server host */
                    'dtmfmode': 'rfc2833',
                    'name': peer_name,
                    't38': 'yes',
                    'options_keepalive': 'no',
                    'username': ''};

    newpeer.view = !newpeer.view;
    ctrl.add_codecs_options(newpeer);
    ctrl.sip.peers.push(newpeer);
    ctrl.check_invalid_config();
  };

  function update_disabled_codecs_list(disabled_list, codec_to_remove) {
    // disabled_list is a Arry with objects, then we need looking for codec in all positions.
    for (var j = 0; j < disabled_list.length; j++) {
      if (disabled_list[j].label == codec_to_remove) {
        disabled_list.splice(j, 1);
      }
    }
  };

  ctrl.add_codecs_options = function(peer) {
    peer.codecs_models = {selected: null,
                          lists: {"Disabled": [{label: "alaw"}, {label: "ulaw"}, {label: "g729"}, {label: "g726"}, {label: "g723"}],
                                  "Enabled": []
                                 }
                          }

    // This peer already exists, then get the current codecs
    for (var i = 0; i < peer.codecs.length; i++) {
      peer.codecs_models.lists.Enabled.push({label: peer.codecs[i]});
      update_disabled_codecs_list(peer.codecs_models.lists.Disabled, peer.codecs[i]);
    }
  };

  /**
   * This function is used by button 'confirm' in web page.
   * This button create a new peer, after we do push to
   * main object 'ctrl.sip.peers'.
   *
   * @function confirm_newpeer
   * @memberof SIPController
   */
  ctrl.confirm_newpeer = function(peer) {
    ctrl.sip.peers.push(peer);
  };

  /**
   * Check if some configuration is invalid. If true, then disable the
   * button to 'save and apply' config and show the 'danger message'.
   *
   * @function __check_invalid_config
   * @memberof SIPController
   */
  ctrl.check_invalid_config = function() {
    ctrl.invalid_config = false;
    
    if (ctrl.invalid_bindport) {
      ctrl.invalid_config = true;
      ctrl.alert_danger = true;
      return;
    }

    for (var i = 0; i < ctrl.sip.peers.length; i++) {
      var peer = ctrl.sip.peers[i]
      if (peer.invalid_port ||peer.invalid_peer_name || peer.invalid_host_ip || peer.invalid_username) {
        ctrl.invalid_config = true;
        break;
      }
    }

    ctrl.alert_danger = ctrl.invalid_config;
  };

  /**
   * Check if input to Bind Port is a valid value
   * Update the state of the variable ctrl.invalid_bindport (true or false)
   * @function validateBindPort
   * @memberof SIPController
   */
  ctrl.validateBindPort = function(port) {
    ctrl.invalid_bindport = port <= 0 || port >= 65536;

    ctrl.check_invalid_config();
  };

  /**
   * Check if input to Port is a valid value
   * Update the state of the variable valid_port (true or false) inside of peer
   * @function validatePort
   * @memberof SIPController
   */
  ctrl.validatePort = function(peer) {
    peer.invalid_port = peer.port <= 0 || peer.port >= 65536;

    ctrl.check_invalid_config();
  };

  ctrl.validatePeerName = function(peer) {
    var regex = /^\w+$/;

    peer.invalid_peer_name = false;
    if (!regex.test(peer.name) || peer.name == "general") {
      peer.invalid_peer_name = true;
    } else {
      // Check if this name already exists in another peer
      var count = 0;
      for (var i = 0; i < ctrl.sip.peers.length; i++) {
        if (ctrl.sip.peers[i].name == peer.name)
          count++;
      }
      
      if (count > 1) {
        rule.invalid_peer_name = true;
      }
    }
    ctrl.check_invalid_config();
  };

  /**
   * Validate HOST.
   * Situations that we accept:
   * If register is enabled, then the host can be
   * any address (IP or name), but it can not be
   * a string with space, "," or ";" and we don't
   * accept "dynamic".
   *
   * If register is disabled, then the situation
   * is similar, but in this case we accept
   * "dynamic" word.
   *
   * @function validateIP
   * @memberof SIPController
   */
  ctrl.validateServerIP = function(peer) {
    var regex = /^[a-zA-Z0-9]+[a-zA-Z0-9\.\-]*$/;

    if (!regex.test(peer.host)) {
      // Check if host is valid
      peer.invalid_host_ip = true;
    } else if ((peer.register == "yes") && peer.host == "dynamic") {
      // If register is disabled, host can not be 'dynamic'
      peer.invalid_host_ip = true;
    } else {
      peer.invalid_host_ip = false;
    }
    
    ctrl.check_invalid_config();
  };

  /**
   * Validade Username to register
   * If register is enabled, then we need a valid username.
   * It can not be a string with space, ",", ";" or empty.
   *
   * @function validateUsername
   * @memberof SIPController
   */
  ctrl.validateUsername = function(peer) {
    var regex = /^[^\s;,]*$/;

    if (!regex.test(peer.username)) {
      // Check if username is valid
      peer.invalid_username = true;
    } else if (peer.register == "yes" && peer.username.length == 0) {
      // If register is enabled, user must not be empty
      peer.invalid_username = true;
    } else {
      peer.invalid_username = false;
    }
    
    ctrl.check_invalid_config();
  };

  /**
   * When the register is set to "Enabled" we need
   * to check some variables too.
   *
   * @function validateRegister
   * @memberof SIPController
   */
  ctrl.validateRegister = function(peer) {
    ctrl.validateServerIP(peer);
    ctrl.validateUsername(peer);
  };

}]);
